--[[
Update 22.11.2024, 00:00
> DE - Kompatibilitätsvorbereitung zu 'Courseplay' (https://github.com/Courseplay/Courseplay_FS25)
Für die Vorbereitung der Kompatibilität mit 'Courseplay' wurde ein Skript von 'alpepi' verwendet,
das 'Nahtlose Kontrolle' mit 'Courseplay' kompatibel macht, sobald 'Courseplay' verfügbar ist.
Da sich 'Courseplay' jedoch noch in der Entwicklung befindet, kann es passieren, dass der Skript
nicht immer wie erwartet funktioniert.

> EN - Compatibility Preparation for 'Courseplay' (https://github.com/Courseplay/Courseplay_FS25)
To prepare for compatibility with 'Courseplay,' a script by 'alpepi" was used, enabling 'Seamless Control'
to work with 'Courseplay' once it becomes available. However, since 'Courseplay' is still under development,
the script may not always function as expected.
]]
-- Verweis auf die Kompatiblitätsdatei zu 'Courseplay' / Reference to the compatibility file for 'Courseplay'
source(g_currentModDirectory .. "courseplayCompatibility.lua")

local function ccCheck(ajv)
    if  ajv:getCruiseControlState() == Drivable.CRUISECONTROL_STATE_OFF or ajv:getAIFieldWorkerIsTurning()
        or ajv:getDrivingDirection() <= 0 or (ajv.spec_aiFieldWorker.aiDriveParams.maxSpeed ~= ajv:getSpeedLimit(true) and ajv.spec_aiFieldWorker.aiDriveParams.valid)
    then
        return false
    else
        return true
    end
end

local function isUnfoldedCheck(v, specF)
    if  specF == nil or #specF.foldingParts == 0
        or not v:getIsFoldMiddleAllowed() or not v:getIsUnfolded()
        or specF.foldMoveDirection == -1*specF.turnOnFoldDirection or specF.foldAnimTime == specF.foldMiddleAnimTime
    then
        return false
    else
        return true
    end
end

local function toggleAIVehicleOverwrite(aijobvehicle, superFunc)
    if aijobvehicle == nil then
        return(superFunc(aijobvehicle))
    end

    local turnOnCruiseCheck = nil
    local vehicles = nil
    local implements = nil
    local beforeVehicleStates = {}
    local beforeImplementStates = {}

    if aijobvehicle:getIsControlled() and aijobvehicle:getIsAIActive() then
        turnOnCruiseCheck = ccCheck(aijobvehicle)

        vehicles = aijobvehicle:getChildVehicles()
        if vehicles ~= nil then
            for i, vehicle in ipairs(vehicles) do
                beforeVehicleStates[i] = {
                    isUnfolded = isUnfoldedCheck(vehicle, vehicle.spec_foldable),
                    isTurnedOn = vehicle.spec_turnOnVehicle ~= nil and vehicle:getIsTurnedOn()
                }
            end
        end

        implements = aijobvehicle:getAttachedImplements()
        if implements ~= nil then
            for _, implement in ipairs(implements) do
            beforeImplementStates[implement.jointDescIndex] = {
                    isLowered = implement.object.spec_attachable.attacherJoint.allowsLowering and implement.object:getIsLowered(true)
                }
            end
        end
    end

    local returnValue = superFunc(aijobvehicle)

    if aijobvehicle:getIsControlled() and (not aijobvehicle:getIsAIActive()) then
        if turnOnCruiseCheck then
            aijobvehicle:setCruiseControlMaxSpeed(aijobvehicle:getCruiseControlSpeed())
            aijobvehicle:setCruiseControlState(Drivable.CRUISECONTROL_STATE_ACTIVE, false)
			-- Kompatibilitätsfunktion zu 'Courseplay'
            courseplayCompatibility.cpKeepCC(aijobvehicle)
        end

        if vehicles ~= nil then
            for i, vehicle in ipairs(vehicles) do
                if beforeVehicleStates[i].isTurnedOn then
                    vehicle:setIsTurnedOn(true, false)
                end

                if beforeVehicleStates[i].isUnfolded then
                    vehicle:setFoldState(vehicle.spec_foldable.turnOnFoldDirection , false, false)
                end
            end
        end

        if implements ~= nil then
            for _, implement in ipairs(implements) do
                if beforeImplementStates[implement.jointDescIndex].isLowered then
                    implement.object:setLoweredAll(true, implement.jointDescIndex)

                    if implement.object.spec_attachable.lowerAnimation ~= nil and implement.object.playAnimation ~= nil then
                        local spec = implement.object.spec_attachable
                        implement.object:playAnimation(spec.lowerAnimation, spec.lowerAnimationSpeed, implement.object:getAnimationTime(spec.lowerAnimation), false)
                    end
                end
            end
        end
    end

    return returnValue
end

AIJobVehicle.toggleAIVehicle = Utils.overwrittenFunction(AIJobVehicle.toggleAIVehicle, toggleAIVehicleOverwrite)